@@ -1,5 +1,7 @@
 from __future__ import annotations
 
+import dataclasses
+import logging
 import os
 import re
 import warnings
@@ -8,166 +10,150 @@ from datetime import datetime
 from datetime import timezone
 from typing import Any
 from typing import Callable
-from typing import cast
-from typing import Iterator
-from typing import List
 from typing import Match
-from typing import overload
-from typing import Tuple
 from typing import TYPE_CHECKING
 
+from . import _entrypoints
+from . import _modify_version
+
 if TYPE_CHECKING:
     from typing_extensions import Concatenate
+    from typing_extensions import ParamSpec
 
-    from . import _types as _t
+    _P = ParamSpec("_P")
+
+from typing import TypedDict
+
+
+from ._version_cls import Version as PkgVersion, _VersionT
+from . import _version_cls as _v
+from . import _config
+
+log = logging.getLogger(__name__)
 
-from ._version_cls import Version as PkgVersion
-from .config import Configuration
-from .config import _VersionT
-from .utils import trace
 
 SEMVER_MINOR = 2
 SEMVER_PATCH = 3
 SEMVER_LEN = 3
 
 
+class _TagDict(TypedDict):
+    version: str
+    prefix: str
+    suffix: str
+
+
 def _parse_version_tag(
-    tag: str | object, config: Configuration
-) -> dict[str, str] | None:
-    tagstring = tag if isinstance(tag, str) else str(tag)
-    match = config.tag_regex.match(tagstring)
+    tag: str | object, config: _config.Configuration
+) -> _TagDict | None:
+    match = config.tag_regex.match(str(tag))
 
-    result = None
     if match:
-        key: str | int
-        if len(match.groups()) == 1:
-            key = 1
-        else:
-            key = "version"
+        key: str | int = 1 if len(match.groups()) == 1 else "version"
+        full = match.group(0)
+        log.debug("%r %r %s", tag, config.tag_regex, match)
+        log.debug(
+            "key %s data %s, %s, %r", key, match.groupdict(), match.groups(), full
+        )
+        result = _TagDict(
+            version=match.group(key),
+            prefix=full[: match.start(key)],
+            suffix=full[match.end(key) :],
+        )
 
-        result = {
-            "version": match.group(key),
-            "prefix": match.group(0)[: match.start(key)],
-            "suffix": match.group(0)[match.end(key) :],
-        }
+        log.debug("tag %r parsed to %r", tag, result)
+        assert result["version"]
+        return result
+    else:
+        log.debug("tag %r did not parse", tag)
 
-    trace(f"tag '{tag}' parsed to {result}")
-    return result
+        return None
 
 
 def callable_or_entrypoint(group: str, callable_or_name: str | Any) -> Any:
-    trace("ep", (group, callable_or_name))
+    log.debug("ep %r %r", group, callable_or_name)
 
     if callable(callable_or_name):
         return callable_or_name
     from ._entrypoints import iter_entry_points
 
     for ep in iter_entry_points(group, callable_or_name):
-        trace("ep found:", ep.name)
+        log.debug("ep found: %s", ep.name)
         return ep.load()
 
 
 def tag_to_version(
-    tag: _VersionT | str, config: Configuration | None = None
+    tag: _VersionT | str, config: _config.Configuration
 ) -> _VersionT | None:
     """
     take a tag that might be prefixed with a keyword and return only the version part
-    :param config: optional configuration object
     """
-    trace("tag", tag)
+    log.debug("tag %s", tag)
 
-    if not config:
-        config = Configuration()
-
-    tagdict = _parse_version_tag(tag, config)
-    if not isinstance(tagdict, dict) or not tagdict.get("version", None):
+    tag_dict = _parse_version_tag(tag, config)
+    if tag_dict is None or not tag_dict.get("version", None):
         warnings.warn(f"tag {tag!r} no version found")
         return None
 
-    version_str = tagdict["version"]
-    trace("version pre parse", version_str)
+    version_str = tag_dict["version"]
+    log.debug("version pre parse %s", version_str)
 
-    if tagdict.get("suffix", ""):
-        warnings.warn(
-            "tag {!r} will be stripped of its suffix '{}'".format(
-                tag, tagdict["suffix"]
-            )
-        )
+    if suffix := tag_dict.get("suffix", ""):
+        warnings.warn(f"tag {tag!r} will be stripped of its suffix {suffix!r}")
 
-    version = config.version_cls(version_str)
-    trace("version", repr(version))
+    version: _VersionT = config.version_cls(version_str)
+    log.debug("version=%r", version)
 
     return version
 
 
-def tags_to_versions(
-    tags: list[str], config: Configuration | None = None
-) -> list[_VersionT]:
-    """
-    take tags that might be prefixed with a keyword and return only the version part
-    :param tags: an iterable of tags
-    :param config: optional configuration object
-    """
-    result: list[_VersionT] = []
-    for tag in tags:
-        parsed = tag_to_version(tag, config=config)
-        if parsed:
-            result.append(parsed)
-    return result
+def _source_epoch_or_utc_now() -> datetime:
+    if "SOURCE_DATE_EPOCH" in os.environ:
+        date_epoch = int(os.environ["SOURCE_DATE_EPOCH"])
+        return datetime.fromtimestamp(date_epoch, timezone.utc)
+    else:
+        return datetime.now(timezone.utc)
 
 
+@dataclasses.dataclass
 class ScmVersion:
-    def __init__(
-        self,
-        tag_version: Any,
-        config: Configuration,
-        distance: int | None = None,
-        node: str | None = None,
-        dirty: bool = False,
-        preformatted: bool = False,
-        branch: str | None = None,
-        node_date: date | None = None,
-        **kw: object,
-    ):
-        if kw:
-            trace("unknown args", kw)
-        self.tag = tag_version
-        if dirty and distance is None:
-            distance = 0
-        self.distance = distance
-        self.node = node
-        self.node_date = node_date
-        if "SOURCE_DATE_EPOCH" in os.environ:
-            date_epoch = int(os.environ["SOURCE_DATE_EPOCH"])
-            self.time = datetime.fromtimestamp(date_epoch, timezone.utc)
-        else:
-            self.time = datetime.now(timezone.utc)
-        self._extra = kw
-        self.dirty = dirty
-        self.preformatted = preformatted
-        self.branch = branch
-        self.config = config
+    """represents a parsed version from scm"""
 
-    @property
-    def extra(self) -> dict[str, Any]:
-        warnings.warn(
-            "ScmVersion.extra is deprecated and will be removed in future",
-            category=DeprecationWarning,
-            stacklevel=2,
-        )
-        return self._extra
+    tag: _v.Version | _v.NonNormalizedVersion | str
+    """the related tag or preformatted version string"""
+    config: _config.Configuration
+    """the configuration used to parse the version"""
+    distance: int = 0
+    """the number of commits since the tag"""
+    node: str | None = None
+    """the shortened node id"""
+    dirty: bool = False
+    """whether the working copy had uncommitted changes"""
+    preformatted: bool = False
+    """whether the version string was preformatted"""
+    branch: str | None = None
+    """the branch name if any"""
+    node_date: date | None = None
+    """the date of the commit if available"""
+    time: datetime = dataclasses.field(default_factory=_source_epoch_or_utc_now)
+    """the current time or source epoch time
+    only set for unit-testing version schemes
+    for real usage it must be `now(utc)` or `SOURCE_EPOCH`
+    """
 
     @property
     def exact(self) -> bool:
-        return self.distance is None
+        """returns true checked out exactly on a tag and no local changes apply"""
+        return self.distance == 0 and not self.dirty
 
     def __repr__(self) -> str:
-        return self.format_with(
-            "<ScmVersion {tag} dist={distance} "
-            "node={node} dirty={dirty} branch={branch}>"
+        return (
+            f"<ScmVersion {self.tag} dist={self.distance} "
+            f"node={self.node} dirty={self.dirty} branch={self.branch}>"
         )
 
     def format_with(self, fmt: str, **kw: object) -> str:
+        """format a given format string with attributes of this object"""
         return fmt.format(
             time=self.time,
             tag=self.tag,
@@ -180,25 +166,29 @@ class ScmVersion:
         )
 
     def format_choice(self, clean_format: str, dirty_format: str, **kw: object) -> str:
+        """given `clean_format` and `dirty_format`
+
+        choose one based on `self.dirty` and format it using `self.format_with`"""
+
         return self.format_with(dirty_format if self.dirty else clean_format, **kw)
 
     def format_next_version(
         self,
-        guess_next: Callable[Concatenate[ScmVersion, _t.P], str],
+        guess_next: Callable[Concatenate[ScmVersion, _P], str],
         fmt: str = "{guessed}.dev{distance}",
-        *k: _t.P.args,  # type: ignore
-        **kw: _t.P.kwargs,  # type: ignore
+        *k: _P.args,
+        **kw: _P.kwargs,
     ) -> str:
         guessed = guess_next(self, *k, **kw)
         return self.format_with(fmt, guessed=guessed)
 
 
 def _parse_tag(
-    tag: _VersionT | str, preformatted: bool, config: Configuration | None
+    tag: _VersionT | str, preformatted: bool, config: _config.Configuration
 ) -> _VersionT | str:
     if preformatted:
         return tag
-    elif config is None or not isinstance(tag, config.version_cls):
+    elif not isinstance(tag, config.version_cls):
         version = tag_to_version(tag, config)
         assert version is not None
         return version
@@ -208,23 +198,17 @@ def _parse_tag(
 
 def meta(
     tag: str | _VersionT,
-    distance: int | None = None,
+    *,
+    distance: int = 0,
     dirty: bool = False,
     node: str | None = None,
     preformatted: bool = False,
     branch: str | None = None,
-    config: Configuration | None = None,
+    config: _config.Configuration,
     node_date: date | None = None,
-    **kw: Any,
 ) -> ScmVersion:
-    if not config:
-        warnings.warn(
-            "meta invoked without explicit configuration,"
-            " will use defaults where required."
-        )
-        config = Configuration()
     parsed_version = _parse_tag(tag, preformatted, config)
-    trace("version", tag, "->", parsed_version)
+    log.info("version %s -> %s", tag, parsed_version)
     assert parsed_version is not None, "Can't parse version %s" % tag
     return ScmVersion(
         parsed_version,
@@ -235,58 +219,12 @@ def meta(
         branch=branch,
         config=config,
         node_date=node_date,
-        **kw,
     )
 
 
 def guess_next_version(tag_version: ScmVersion) -> str:
-    version = _strip_local(str(tag_version.tag))
-    return _bump_dev(version) or _bump_regex(version)
-
-
-def _dont_guess_next_version(tag_version: ScmVersion) -> str:
-    version = _strip_local(str(tag_version.tag))
-    return _bump_dev(version) or _add_post(version)
-
-
-def _strip_local(version_string: str) -> str:
-    public, sep, local = version_string.partition("+")
-    return public
-
-
-def _add_post(version: str) -> str:
-    if "post" in version:
-        raise ValueError(
-            f"{version} already is a post release, refusing to guess the update"
-        )
-    return f"{version}.post1"
-
-
-def _bump_dev(version: str) -> str | None:
-    if ".dev" not in version:
-        return None
-
-    prefix, tail = version.rsplit(".dev", 1)
-    if tail != "0":
-        raise ValueError(
-            "choosing custom numbers for the `.devX` distance "
-            "is not supported.\n "
-            f"The {version} can't be bumped\n"
-            "Please drop the tag or create a new supported one ending in .dev0"
-        )
-    return prefix
-
-
-def _bump_regex(version: str) -> str:
-    match = re.match(r"(.*?)(\d+)$", version)
-    if match is None:
-        raise ValueError(
-            "{version} does not end with a number to bump, "
-            "please correct or use a custom version scheme".format(version=version)
-        )
-    else:
-        prefix, tail = match.groups()
-        return "%s%d" % (prefix, int(tail) + 1)
+    version = _modify_version.strip_local(str(tag_version.tag))
+    return _modify_version._bump_dev(version) or _modify_version._bump_regex(version)
 
 
 def guess_next_dev_version(version: ScmVersion) -> str:
@@ -302,7 +240,7 @@ def guess_next_simple_semver(
     try:
         parts = [int(i) for i in str(version.tag).split(".")[:retain]]
     except ValueError:
-        raise ValueError(f"{version} can't be parsed as numeric version")
+        raise ValueError(f"{version} can't be parsed as numeric version") from None
     while len(parts) < retain:
         parts.append(0)
     if increment:
@@ -353,8 +291,8 @@ def release_branch_semver_version(version: ScmVersion) -> str:
 
 def release_branch_semver(version: ScmVersion) -> str:
     warnings.warn(
-        "release_branch_semver is deprecated and will be removed in future. "
-        + "Use release_branch_semver_version instead",
+        "release_branch_semver is deprecated and will be removed in the future. "
+        "Use release_branch_semver_version instead",
         category=DeprecationWarning,
         stacklevel=2,
     )
@@ -365,18 +303,22 @@ def no_guess_dev_version(version: ScmVersion) -> str:
     if version.exact:
         return version.format_with("{tag}")
     else:
-        return version.format_next_version(_dont_guess_next_version)
+        return version.format_next_version(_modify_version._dont_guess_next_version)
+
+
+_DATE_REGEX = re.compile(
+    r"""
+    ^(?P<date>
+        (?P<prefix>[vV]?)
+        (?P<year>\d{2}|\d{4})(?:\.\d{1,2}){2})
+        (?:\.(?P<patch>\d*))?$
+    """,
+    re.VERBOSE,
+)
 
 
 def date_ver_match(ver: str) -> Match[str] | None:
-    match = re.match(
-        (
-            r"^(?P<date>(?P<year>\d{2}|\d{4})(?:\.\d{1,2}){2})"
-            r"(?:\.(?P<patch>\d*)){0,1}?$"
-        ),
-        ver,
-    )
-    return match
+    return _DATE_REGEX.match(ver)
 
 
 def guess_next_date_ver(
@@ -403,13 +345,21 @@ def guess_next_date_ver(
         # deduct date format if not provided
         if date_fmt is None:
             date_fmt = "%Y.%m.%d" if len(match.group("year")) == 4 else "%y.%m.%d"
-    today = datetime.now(timezone.utc).date()
+        if prefix := match.group("prefix"):
+            if not date_fmt.startswith(prefix):
+                date_fmt = prefix + date_fmt
+
+    today = version.time.date()
     head_date = node_date or today
     # compute patch
     if match is None:
         tag_date = today
     else:
-        tag_date = datetime.strptime(match.group("date"), date_fmt).date()
+        tag_date = (
+            datetime.strptime(match.group("date"), date_fmt)
+            .replace(tzinfo=timezone.utc)
+            .date()
+        )
     if tag_date == head_date:
         patch = "0" if match is None else (match.group("patch") or "0")
         patch = int(patch) + 1
@@ -417,9 +367,8 @@ def guess_next_date_ver(
         if tag_date > head_date and match is not None:
             # warn on future times
             warnings.warn(
-                "your previous tag  ({}) is ahead your node date ({})".format(
-                    tag_date, head_date
-                )
+                f"your previous tag  ({tag_date})"
+                f" is ahead your node date ({head_date})"
             )
         patch = 0
     next_version = "{node_date:{date_fmt}}.{patch}".format(
@@ -450,31 +399,19 @@ def calver_by_date(version: ScmVersion) -> str:
     )
 
 
-def _format_local_with_time(version: ScmVersion, time_format: str) -> str:
-
-    if version.exact or version.node is None:
-        return version.format_choice(
-            "", "+d{time:{time_format}}", time_format=time_format
-        )
-    else:
-        return version.format_choice(
-            "+{node}", "+{node}.d{time:{time_format}}", time_format=time_format
-        )
-
-
 def get_local_node_and_date(version: ScmVersion) -> str:
-    return _format_local_with_time(version, time_format="%Y%m%d")
+    return _modify_version._format_local_with_time(version, time_format="%Y%m%d")
 
 
-def get_local_node_and_timestamp(version: ScmVersion, fmt: str = "%Y%m%d%H%M%S") -> str:
-    return _format_local_with_time(version, time_format=fmt)
+def get_local_node_and_timestamp(version: ScmVersion) -> str:
+    return _modify_version._format_local_with_time(version, time_format="%Y%m%d%H%M%S")
 
 
 def get_local_dirty_tag(version: ScmVersion) -> str:
     return version.format_choice("", "+dirty")
 
 
-def get_no_local_node(_: Any) -> str:
+def get_no_local_node(version: ScmVersion) -> str:
     return ""
 
 
@@ -485,79 +422,19 @@ def postrelease_version(version: ScmVersion) -> str:
         return version.format_with("{tag}.post{distance}")
 
 
-def _get_ep(group: str, name: str) -> Any | None:
-    from ._entrypoints import iter_entry_points
-
-    for ep in iter_entry_points(group, name):
-        trace("ep found:", ep.name)
-        return ep.load()
-    else:
-        return None
-
-
-def _iter_version_schemes(
-    entrypoint: str,
-    scheme_value: str
-    | list[str]
-    | tuple[str, ...]
-    | Callable[[ScmVersion], str]
-    | None,
-    _memo: set[object] | None = None,
-) -> Iterator[Callable[[ScmVersion], str]]:
-    if _memo is None:
-        _memo = set()
-    if isinstance(scheme_value, str):
-        scheme_value = cast(
-            'str|List[str]|Tuple[str, ...]|Callable[["ScmVersion"], str]|None',
-            _get_ep(entrypoint, scheme_value),
-        )
-
-    if isinstance(scheme_value, (list, tuple)):
-        for variant in scheme_value:
-            if variant not in _memo:
-                _memo.add(variant)
-                yield from _iter_version_schemes(entrypoint, variant, _memo=_memo)
-    elif callable(scheme_value):
-        yield scheme_value
-
-
-@overload
-def _call_version_scheme(
-    version: ScmVersion, entypoint: str, given_value: str, default: str
-) -> str:
-    ...
-
-
-@overload
-def _call_version_scheme(
-    version: ScmVersion, entypoint: str, given_value: str, default: None
-) -> str | None:
-    ...
-
-
-def _call_version_scheme(
-    version: ScmVersion, entypoint: str, given_value: str, default: str | None
-) -> str | None:
-    for scheme in _iter_version_schemes(entypoint, given_value):
-        result = scheme(version)
-        if result is not None:
-            return result
-    return default
-
-
-def format_version(version: ScmVersion, **config: Any) -> str:
-    trace("scm version", version)
-    trace("config", config)
+def format_version(version: ScmVersion) -> str:
+    log.debug("scm version %s", version)
+    log.debug("config %s", version.config)
     if version.preformatted:
         assert isinstance(version.tag, str)
         return version.tag
-    main_version = _call_version_scheme(
-        version, "setuptools_scm.version_scheme", config["version_scheme"], None
+    main_version = _entrypoints._call_version_scheme(
+        version, "setuptools_scm.version_scheme", version.config.version_scheme, None
     )
-    trace("version", main_version)
+    log.debug("version %s", main_version)
     assert main_version is not None
-    local_version = _call_version_scheme(
-        version, "setuptools_scm.local_scheme", config["local_scheme"], "+unknown"
+    local_version = _entrypoints._call_version_scheme(
+        version, "setuptools_scm.local_scheme", version.config.local_scheme, "+unknown"
     )
-    trace("local_version", local_version)
+    log.debug("local_version %s", local_version)
     return main_version + local_version
