@@ -1,26 +1,33 @@
 from __future__ import annotations
 
 import os
-import shutil
 import sys
+from datetime import date
 from pathlib import Path
 
 import pytest
 
 import setuptools_scm
+from setuptools_scm import Configuration
 from setuptools_scm import dump_version
-from setuptools_scm.config import Configuration
-from setuptools_scm.utils import data_from_mime
-from setuptools_scm.utils import do
+from setuptools_scm._run_cmd import run
+from setuptools_scm.integration import data_from_mime
+from setuptools_scm.version import meta
 from setuptools_scm.version import ScmVersion
 from testing.wd_wrapper import WorkDir
 
 
-@pytest.mark.parametrize("cmd", ["ls", "dir"])
-def test_do(cmd: str, tmp_path: Path) -> None:
-    if not shutil.which(cmd):
-        pytest.skip(f"{cmd} not found")
-    do(cmd, cwd=tmp_path)
+c = Configuration()
+
+template = """\
+__version__ = version = {version!r}
+__version_tuple__ = version_tuple = {version_tuple!r}
+__sha__ = {scm_version.node!r}
+"""
+
+
+def test_run_plain(tmp_path: Path) -> None:
+    run([sys.executable, "-c", "print(1)"], cwd=tmp_path)
 
 
 def test_data_from_mime(tmp_path: Path) -> None:
@@ -34,7 +41,7 @@ def test_data_from_mime(tmp_path: Path) -> None:
 def test_version_from_pkginfo(wd: WorkDir) -> None:
     wd.write("PKG-INFO", "Version: 0.1")
 
-    assert wd.version == "0.1"
+    assert wd.get_version() == "0.1"
 
     # replicate issue 167
     assert wd.get_version(version_scheme="1.{0.distance}.0".format) == "0.1"
@@ -49,7 +56,7 @@ def assert_root(monkeypatch: pytest.MonkeyPatch, expected_root: str) -> None:
         assert config.absolute_root == expected_root
         return ScmVersion("1.0", config=config)
 
-    monkeypatch.setattr(setuptools_scm, "_do_parse", assertion)
+    monkeypatch.setattr(setuptools_scm._get_version_impl, "parse_version", assertion)
 
 
 def test_root_parameter_creation(monkeypatch: pytest.MonkeyPatch) -> None:
@@ -57,11 +64,6 @@ def test_root_parameter_creation(monkeypatch: pytest.MonkeyPatch) -> None:
     setuptools_scm.get_version()
 
 
-def test_version_from_scm(wd: WorkDir) -> None:
-    with pytest.warns(DeprecationWarning, match=".*version_from_scm.*"):
-        setuptools_scm.version_from_scm(str(wd))
-
-
 def test_root_parameter_pass_by(
     monkeypatch: pytest.MonkeyPatch, tmp_path: Path
 ) -> None:
@@ -81,8 +83,8 @@ def test_parentdir_prefix(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> No
 setup(use_scm_version={"parentdir_prefix_version": "projectname-"})
 """
     )
-    res = do([sys.executable, "setup.py", "--version"], p)
-    assert res == "12.34"
+    res = run([sys.executable, "setup.py", "--version"], p)
+    assert res.stdout == "12.34"
 
 
 def test_fallback(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
@@ -94,8 +96,8 @@ def test_fallback(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
 setup(use_scm_version={"fallback_version": "12.34"})
 """
     )
-    res = do([sys.executable, "setup.py", "--version"], p)
-    assert res == "12.34"
+    res = run([sys.executable, "setup.py", "--version"], p)
+    assert res.stdout == "12.34"
 
 
 def test_empty_pretend_version(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
@@ -108,8 +110,8 @@ def test_empty_pretend_version(tmp_path: Path, monkeypatch: pytest.MonkeyPatch)
 setup(use_scm_version={"fallback_version": "12.34"})
 """
     )
-    res = do([sys.executable, "setup.py", "--version"], p)
-    assert res == "12.34"
+    res = run([sys.executable, "setup.py", "--version"], p)
+    assert res.stdout == "12.34"
 
 
 def test_empty_pretend_version_named(
@@ -126,15 +128,22 @@ def test_empty_pretend_version_named(
 setup(name="myscm", use_scm_version={"fallback_version": "12.34"})
 """
     )
-    res = do([sys.executable, "setup.py", "--version"], p)
-    assert res == "12.34"
+    res = run([sys.executable, "setup.py", "--version"], p)
+    assert res.stdout == "12.34"
+
+
+def test_get_version_blank_tag_regex() -> None:
+    with pytest.warns(
+        DeprecationWarning, match="empty regex for tag regex is invalid, using default"
+    ):
+        setuptools_scm.get_version(tag_regex="")
 
 
 @pytest.mark.parametrize(
     "version", ["1.0", "1.2.3.dev1+ge871260", "1.2.3.dev15+ge871260.d20180625", "2345"]
 )
 def test_pretended(version: str, monkeypatch: pytest.MonkeyPatch) -> None:
-    monkeypatch.setenv(setuptools_scm.PRETEND_KEY, version)
+    monkeypatch.setenv(setuptools_scm._overrides.PRETEND_KEY, version)
     assert setuptools_scm.get_version() == version
 
 
@@ -157,31 +166,48 @@ def test_root_relative_to(monkeypatch: pytest.MonkeyPatch, tmp_path: Path) -> No
 
 
 def test_dump_version(tmp_path: Path) -> None:
-
-    dump_version(tmp_path, "1.0", "first.txt")
+    version = "1.0"
+    scm_version = meta(version, config=c)
+    dump_version(tmp_path, version, "first.txt", scm_version=scm_version)
 
     def read(name: str) -> str:
         return tmp_path.joinpath(name).read_text()
 
     assert read("first.txt") == "1.0"
 
-    dump_version(tmp_path, "1.0.dev42", "first.py")
+    version = "1.0.dev42"
+    scm_version = meta("1.0", distance=42, config=c)
+    dump_version(tmp_path, version, "first.py", scm_version=scm_version)
     lines = read("first.py").splitlines()
-    assert "__version__ = version = '1.0.dev42'" in lines
-    assert "__version_tuple__ = version_tuple = (1, 0, 'dev42')" in lines
+    assert lines[-2:] == [
+        "__version__ = version = '1.0.dev42'",
+        "__version_tuple__ = version_tuple = (1, 0, 'dev42')",
+    ]
 
-    dump_version(tmp_path, "1.0.1+g4ac9d2c", "second.py")
+    version = "1.0.1+g4ac9d2c"
+    scm_version = meta("1.0.1", node="g4ac9d2c", config=c)
+    dump_version(
+        tmp_path, version, "second.py", scm_version=scm_version, template=template
+    )
     lines = read("second.py").splitlines()
     assert "__version__ = version = '1.0.1+g4ac9d2c'" in lines
     assert "__version_tuple__ = version_tuple = (1, 0, 1, 'g4ac9d2c')" in lines
+    assert "__sha__ = 'g4ac9d2c'" in lines
 
-    dump_version(tmp_path, "1.2.3.dev18+gb366d8b.d20210415", "third.py")
+    version = "1.2.3.dev18+gb366d8b.d20210415"
+    scm_version = meta(
+        "1.2.3", node="gb366d8b", distance=18, node_date=date(2021, 4, 15), config=c
+    )
+    dump_version(
+        tmp_path, version, "third.py", scm_version=scm_version, template=template
+    )
     lines = read("third.py").splitlines()
     assert "__version__ = version = '1.2.3.dev18+gb366d8b.d20210415'" in lines
     assert (
         "__version_tuple__ = version_tuple = (1, 2, 3, 'dev18', 'gb366d8b.d20210415')"
         in lines
     )
+    assert "__sha__ = 'gb366d8b'" in lines
 
     import ast
 
@@ -200,14 +226,18 @@ def test_custom_version_cls() -> None:
     """Test that `normalize` and `version_cls` work as expected"""
 
     class MyVersion:
-        def __init__(self, tag_str: str):
+        def __init__(self, tag_str: str) -> None:
             self.version = tag_str
 
         def __repr__(self) -> str:
             return f"hello,{self.version}"
 
     # you can not use normalize=False and version_cls at the same time
-    with pytest.raises(ValueError):
+    with pytest.raises(
+        ValueError,
+        match="Providing a custom `version_cls`"
+        " is not permitted when `normalize=False`",
+    ):
         setuptools_scm.get_version(normalize=False, version_cls=MyVersion)
 
     # TODO unfortunately with PRETEND_KEY the preformatted flag becomes True
@@ -215,3 +245,20 @@ def test_custom_version_cls() -> None:
     #  to create a test?
     # monkeypatch.setenv(setuptools_scm.PRETEND_KEY, "1.0.1")
     # assert setuptools_scm.get_version(version_cls=MyVersion) == "1"
+
+
+def test_internal_get_version_warns_for_version_files(tmp_path: Path) -> None:
+    tmp_path.joinpath("PKG-INFO").write_text("Version: 0.1")
+    c = Configuration(root=tmp_path, fallback_root=tmp_path)
+    with pytest.warns(
+        DeprecationWarning,
+        match="force_write_version_files ought to be set,"
+        " presuming the legacy True value",
+    ):
+        ver = setuptools_scm._get_version(c)
+    assert ver == "0.1"
+
+    # force write won't write as no version file is configured
+    assert setuptools_scm._get_version(c, force_write_version_files=False) == ver
+
+    assert setuptools_scm._get_version(c, force_write_version_files=True) == ver
