@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import contextlib
 import os
 import shutil
 import subprocess
@@ -10,22 +11,25 @@ from datetime import timezone
 from os.path import join as opj
 from pathlib import Path
 from textwrap import dedent
+from typing import Generator
 from unittest.mock import Mock
 from unittest.mock import patch
 
 import pytest
 
+import setuptools_scm._file_finders
 from .conftest import DebugMode
 from .wd_wrapper import WorkDir
 from setuptools_scm import Configuration
-from setuptools_scm import format_version
 from setuptools_scm import git
-from setuptools_scm import integration
 from setuptools_scm import NonNormalizedVersion
-from setuptools_scm.file_finder_git import git_find_files
+from setuptools_scm._file_finders.git import git_find_files
+from setuptools_scm._run_cmd import CommandNotFoundError
+from setuptools_scm._run_cmd import CompletedProcess
+from setuptools_scm._run_cmd import has_command
+from setuptools_scm._run_cmd import run
 from setuptools_scm.git import archival_to_version
-from setuptools_scm.utils import do
-from setuptools_scm.utils import has_command
+from setuptools_scm.version import format_version
 
 pytestmark = pytest.mark.skipif(
     not has_command("git", warn=False), reason="git executable not found"
@@ -46,7 +50,7 @@ def wd(wd: WorkDir, monkeypatch: pytest.MonkeyPatch, debug_mode: DebugMode) -> W
 
 
 @pytest.mark.parametrize(
-    "given, tag, number, node, dirty",
+    ("given", "tag", "number", "node", "dirty"),
     [
         ("3.3.1-rc26-0-g9df187b", "3.3.1-rc26", 0, "g9df187b", False),
         ("17.33.0-rc-17-g38c3047c0", "17.33.0-rc", 17, "g38c3047c0", False),
@@ -69,8 +73,8 @@ setup(use_scm_version={"root": "../..",
                        "relative_to": __file__})
 """
     )
-    res = do([sys.executable, "setup.py", "--version"], p)
-    assert res == "0.1.dev0"
+    res = run([sys.executable, "setup.py", "--version"], p)
+    assert res.stdout == "0.1.dev0+d20090213"
 
 
 def test_root_search_parent_directories(
@@ -84,14 +88,18 @@ def test_root_search_parent_directories(
 setup(use_scm_version={"search_parent_directories": True})
 """
     )
-    res = do([sys.executable, "setup.py", "--version"], p)
-    assert res == "0.1.dev0"
+    res = run([sys.executable, "setup.py", "--version"], p)
+    assert res.stdout == "0.1.dev0+d20090213"
 
 
 def test_git_gone(wd: WorkDir, monkeypatch: pytest.MonkeyPatch) -> None:
     monkeypatch.setenv("PATH", str(wd.cwd / "not-existing"))
-    with pytest.raises(EnvironmentError, match="'git' was not found"):
-        git.parse(str(wd.cwd), git.DEFAULT_DESCRIBE)
+
+    wd.write("pyproject.toml", "[tool.setuptools_scm]")
+    with pytest.raises(CommandNotFoundError, match=r"git"):
+        git.parse(wd.cwd, Configuration(), git.DEFAULT_DESCRIBE)
+
+    assert wd.get_version(fallback_version="1.0") == "1.0"
 
 
 @pytest.mark.issue("https://github.com/pypa/setuptools_scm/issues/298")
@@ -105,78 +113,80 @@ def test_file_finder_no_history(wd: WorkDir, caplog: pytest.LogCaptureFixture) -
 
 @pytest.mark.issue("https://github.com/pypa/setuptools_scm/issues/281")
 def test_parse_call_order(wd: WorkDir) -> None:
-    git.parse(str(wd.cwd), git.DEFAULT_DESCRIBE)
+    git.parse(str(wd.cwd), Configuration(), git.DEFAULT_DESCRIBE)
+
+
+def sudo_devnull(
+    args: list[str | os.PathLike[str]], check: bool = False
+) -> subprocess.CompletedProcess[bytes]:
+    """shortcut to run sudo with non-interactive input"""
+    return subprocess.run(
+        ["sudo", *args],
+        stdin=subprocess.DEVNULL,
+        check=check,
+    )
+
+
+@contextlib.contextmanager
+def break_folder_permissions(path: Path) -> Generator[None, None, None]:
+    """break the permissions of a folder for a while"""
+    if not shutil.which("sudo"):
+        pytest.skip("sudo executable not found")
+    original_stat = path.stat()
+
+    proc = sudo_devnull(["chown", "-R", "12345", path])
+    if proc.returncode != 0:
+        pytest.xfail("Failed to change ownership, is passwordless sudo available?")
+
+    try:
+        sudo_devnull(["chmod", "a+r", path], check=True)
+        sudo_devnull(["chgrp", "-R", "12345", path], check=True)
+        yield
+    finally:
+        # Restore the ownership
+        sudo_devnull(["chown", "-R", str(original_stat.st_uid), path], check=True)
+        sudo_devnull(["chgrp", "-R", str(original_stat.st_gid), path], check=True)
 
 
 @pytest.mark.issue("https://github.com/pypa/setuptools_scm/issues/707")
-@pytest.mark.xfail(run=False, reason="This test requires passwordless sudo")
 def test_not_owner(wd: WorkDir) -> None:
-    git_dir = opj(wd.cwd)
-    original_stat = os.stat(git_dir)
-    if not shutil.which("sudo"):
-        pytest.skip("sudo executable not found")
-
-    proc = subprocess.run(
-        ["sudo", "chown", "-R", "12345", git_dir], stdin=subprocess.DEVNULL
-    )
-    if proc.returncode != 0:
-        pytest.skip("Failed to change ownership, is passwordless sudo available?")
-    try:
-        subprocess.run(
-            ["sudo", "chmod", "a+r", git_dir], stdin=subprocess.DEVNULL, check=True
-        )
-        subprocess.run(
-            ["sudo", "chgrp", "-R", "12345", git_dir],
-            stdin=subprocess.DEVNULL,
-            check=True,
-        )
-        assert git.parse(str(wd.cwd))
-    finally:
-        # Restore the ownership
-        subprocess.run(
-            ["sudo", "chown", "-R", str(original_stat.st_uid), git_dir],
-            stdin=subprocess.DEVNULL,
-            check=True,
-        )
-        subprocess.run(
-            ["sudo", "chgrp", "-R", str(original_stat.st_gid), git_dir],
-            stdin=subprocess.DEVNULL,
-            check=True,
-        )
+    with break_folder_permissions(wd.cwd):
+        assert git.parse(str(wd.cwd), Configuration())
 
 
 def test_version_from_git(wd: WorkDir) -> None:
-    assert wd.version == "0.1.dev0"
+    assert wd.get_version() == "0.1.dev0+d20090213"
 
-    parsed = git.parse(str(wd.cwd), git.DEFAULT_DESCRIBE)
-    assert parsed is not None and parsed.branch == "master"
+    parsed = git.parse(str(wd.cwd), Configuration(), git.DEFAULT_DESCRIBE)
+    assert parsed is not None
+    assert parsed.branch in ("master", "main")
 
     wd.commit_testfile()
-    assert wd.version.startswith("0.1.dev1+g")
-    assert not wd.version.endswith("1-")
+    assert wd.get_version().startswith("0.1.dev1+g")
+    assert not wd.get_version().endswith("1-")
 
     wd("git tag v0.1")
-    assert wd.version == "0.1"
+    assert wd.get_version() == "0.1"
 
     wd.write("test.txt", "test2")
-    assert wd.version.startswith("0.2.dev0+g")
+    assert wd.get_version().startswith("0.2.dev0+g")
 
     wd.commit_testfile()
-    assert wd.version.startswith("0.2.dev1+g")
+    assert wd.get_version().startswith("0.2.dev1+g")
 
     wd("git tag version-0.2")
-    assert wd.version.startswith("0.2")
+    assert wd.get_version().startswith("0.2")
 
     wd.commit_testfile()
     wd("git tag version-0.2.post210+gbe48adfpost3+g0cc25f2")
     with pytest.warns(
         UserWarning, match="tag '.*' will be stripped of its suffix '.*'"
     ):
-        assert wd.version.startswith("0.2")
+        assert wd.get_version().startswith("0.2")
 
     wd.commit_testfile()
     wd("git tag 17.33.0-rc")
-    assert wd.version == "17.33.0rc0"
+    assert wd.get_version() == "17.33.0rc0"
 
     # custom normalization
     assert wd.get_version(normalize=False) == "17.33.0-rc"
@@ -226,8 +236,8 @@ def test_git_version_unnormalized_setuptools(
     the version is not normalized in write_to files,
     but still normalized by setuptools for the final dist metadata.
     """
-    monkeypatch.delenv("SETUPTOOLS_SCM_DEBUG")
-
+    # monkeypatch.delenv("SETUPTOOLS_SCM_DEBUG")
+    monkeypatch.chdir(wd.cwd)
     wd.write("setup.py", dedent(setup_py_txt))
 
     # do git operations and tag
@@ -253,9 +263,10 @@ def test_unicode_version_scheme(wd: WorkDir) -> None:
 def test_git_worktree(wd: WorkDir) -> None:
     wd.write("test.txt", "test2")
     # untracked files dont change the state
-    assert wd.version == "0.1.dev0"
+    assert wd.get_version() == "0.1.dev0+d20090213"
+
     wd("git add test.txt")
-    assert wd.version.startswith("0.1.dev0+d")
+    assert wd.get_version().startswith("0.1.dev0+d")
 
 
 @pytest.mark.issue(86)
@@ -268,14 +279,15 @@ def test_git_dirty_notag(
     wd.commit_testfile()
     wd.write("test.txt", "test2")
     wd("git add test.txt")
-    assert wd.version.startswith("0.1.dev1")
+    version = wd.get_version()
+
     if today:
         # the date on the tag is in UTC
         tag = datetime.now(timezone.utc).date().strftime(".d%Y%m%d")
     else:
         tag = ".d20090213"
-    # we are dirty, check for the tag
-    assert tag in wd.version
+    assert version.startswith("0.1.dev1+g")
+    assert version.endswith(tag)
 
 
 @pytest.mark.issue(193)
@@ -285,41 +297,42 @@ def test_git_worktree_support(wd: WorkDir, tmp_path: Path) -> None:
     worktree = tmp_path / "work_tree"
     wd("git worktree add -b work-tree %s" % worktree)
 
-    res = do([sys.executable, "-m", "setuptools_scm", "ls"], cwd=worktree)
-    assert "test.txt" in res
-    assert str(worktree) in res
+    res = run([sys.executable, "-m", "setuptools_scm", "ls"], cwd=worktree)
+    assert "test.txt" in res.stdout
+    assert str(worktree) in res.stdout
 
 
-@pytest.fixture
+@pytest.fixture()
 def shallow_wd(wd: WorkDir, tmp_path: Path) -> Path:
     wd.commit_testfile()
     wd.commit_testfile()
     wd.commit_testfile()
     target = tmp_path / "wd_shallow"
-    do(["git", "clone", "file://%s" % wd.cwd, str(target), "--depth=1"])
+    run(["git", "clone", f"file://{wd.cwd}", target, "--depth=1"], tmp_path, check=True)
     return target
 
 
 def test_git_parse_shallow_warns(
     shallow_wd: Path, recwarn: pytest.WarningsRecorder
 ) -> None:
-    git.parse(str(shallow_wd))
+    git.parse(shallow_wd, Configuration())
+    print(list(recwarn))
     msg = recwarn.pop()
     assert "is shallow and may cause errors" in str(msg.message)
 
 
 def test_git_parse_shallow_fail(shallow_wd: Path) -> None:
     with pytest.raises(ValueError, match="git fetch"):
-        git.parse(str(shallow_wd), pre_parse=git.fail_on_shallow)
+        git.parse(str(shallow_wd), Configuration(), pre_parse=git.fail_on_shallow)
 
 
 def test_git_shallow_autocorrect(
     shallow_wd: Path, recwarn: pytest.WarningsRecorder
 ) -> None:
-    git.parse(str(shallow_wd), pre_parse=git.fetch_on_shallow)
+    git.parse(str(shallow_wd), Configuration(), pre_parse=git.fetch_on_shallow)
     msg = recwarn.pop()
     assert "git fetch was used to rectify" in str(msg.message)
-    git.parse(str(shallow_wd), pre_parse=git.fail_on_shallow)
+    git.parse(str(shallow_wd), Configuration(), pre_parse=git.fail_on_shallow)
 
 
 def test_find_files_stop_at_root_git(wd: WorkDir) -> None:
@@ -327,19 +340,19 @@ def test_find_files_stop_at_root_git(wd: WorkDir) -> None:
     project = wd.cwd / "project"
     project.mkdir()
     project.joinpath("setup.cfg").touch()
-    assert integration.find_files(str(project)) == []
+    assert setuptools_scm._file_finders.find_files(str(project)) == []
 
 
 @pytest.mark.issue(128)
 def test_parse_no_worktree(tmp_path: Path) -> None:
-    ret = git.parse(str(tmp_path))
+    ret = git.parse(str(tmp_path), Configuration(root=str(tmp_path)))
     assert ret is None
 
 
 def test_alphanumeric_tags_match(wd: WorkDir) -> None:
     wd.commit_testfile()
     wd("git tag newstyle-development-started")
-    assert wd.version.startswith("0.1.dev1+g")
+    assert wd.get_version().startswith("0.1.dev1+g")
 
 
 def test_git_archive_export_ignore(
@@ -356,7 +369,7 @@ def test_git_archive_export_ignore(
     wd("git add test1.txt test2.txt")
     wd.commit()
     monkeypatch.chdir(wd.cwd)
-    assert integration.find_files(".") == [opj(".", "test1.txt")]
+    assert setuptools_scm._file_finders.find_files(".") == [opj(".", "test1.txt")]
 
 
 @pytest.mark.issue(228)
@@ -366,7 +379,9 @@ def test_git_archive_subdirectory(wd: WorkDir, monkeypatch: pytest.MonkeyPatch)
     wd("git add foobar")
     wd.commit()
     monkeypatch.chdir(wd.cwd)
-    assert integration.find_files(".") == [opj(".", "foobar", "test1.txt")]
+    assert setuptools_scm._file_finders.find_files(".") == [
+        opj(".", "foobar", "test1.txt")
+    ]
 
 
 @pytest.mark.issue(251)
@@ -378,14 +393,14 @@ def test_git_archive_run_from_subdirectory(
     wd("git add foobar")
     wd.commit()
     monkeypatch.chdir(wd.cwd / "foobar")
-    assert integration.find_files(".") == [opj(".", "test1.txt")]
+    assert setuptools_scm._file_finders.find_files(".") == [opj(".", "test1.txt")]
 
 
 @pytest.mark.issue("https://github.com/pypa/setuptools_scm/issues/728")
 def test_git_branch_names_correct(wd: WorkDir) -> None:
     wd.commit_testfile()
     wd("git checkout -b test/fun")
-    wd_git = git.GitWorkdir(os.fspath(wd.cwd))
+    wd_git = git.GitWorkdir(wd.cwd)
     assert wd_git.get_branch() == "test/fun"
 
 
@@ -413,7 +428,6 @@ def test_not_matching_tags(wd: WorkDir) -> None:
 
 
 @pytest.mark.issue("https://github.com/pypa/setuptools_scm/issues/411")
-@pytest.mark.xfail(reason="https://github.com/pypa/setuptools_scm/issues/449")
 def test_non_dotted_version(wd: WorkDir) -> None:
     wd.commit_testfile()
     wd("git tag apache-arrow-1")
@@ -441,23 +455,23 @@ def test_non_dotted_tag_no_version_match(wd: WorkDir) -> None:
 def test_gitdir(monkeypatch: pytest.MonkeyPatch, wd: WorkDir) -> None:
     """ """
     wd.commit_testfile()
-    normal = wd.version
+    normal = wd.get_version()
     # git hooks set this and break subsequent setuptools_scm unless we clean
     monkeypatch.setenv("GIT_DIR", __file__)
-    assert wd.version == normal
+    assert wd.get_version() == normal
 
 
 def test_git_getdate(wd: WorkDir) -> None:
     # TODO: case coverage for git wd parse
-    today = date.today()
+    today = datetime.now(timezone.utc).date()
 
     def parse_date() -> date:
-        parsed = git.parse(os.fspath(wd.cwd))
+        parsed = git.parse(os.fspath(wd.cwd), Configuration())
         assert parsed is not None
         assert parsed.node_date is not None
         return parsed.node_date
 
-    git_wd = git.GitWorkdir(os.fspath(wd.cwd))
+    git_wd = git.GitWorkdir(wd.cwd)
     assert git_wd.get_head_date() is None
     assert parse_date() == today
 
@@ -466,14 +480,21 @@ def test_git_getdate(wd: WorkDir) -> None:
     assert parse_date() == today
 
 
-def test_git_getdate_badgit(wd: WorkDir) -> None:
+def test_git_getdate_badgit(
+    wd: WorkDir, caplog: pytest.LogCaptureFixture, monkeypatch: pytest.MonkeyPatch
+) -> None:
     wd.commit_testfile()
-    git_wd = git.GitWorkdir(os.fspath(wd.cwd))
-    with patch.object(git_wd, "do_ex", Mock(return_value=("%cI", "", 0))):
+    git_wd = git.GitWorkdir(wd.cwd)
+    fake_date_result = CompletedProcess(args=[], stdout="%cI", stderr="", returncode=0)
+    with patch.object(
+        git,
+        "run_git",
+        Mock(return_value=fake_date_result),
+    ):
         assert git_wd.get_head_date() is None
 
 
-@pytest.fixture
+@pytest.fixture()
 def signed_commit_wd(monkeypatch: pytest.MonkeyPatch, wd: WorkDir) -> WorkDir:
     if not has_command("gpg", args=["--version"], warn=False):
         pytest.skip("gpg executable not found")
@@ -500,14 +521,14 @@ Expire-Date: 0
 
 @pytest.mark.issue("https://github.com/pypa/setuptools_scm/issues/548")
 def test_git_getdate_signed_commit(signed_commit_wd: WorkDir) -> None:
-    today = date.today()
+    today = datetime.now(timezone.utc).date()
     signed_commit_wd.commit_testfile(signed=True)
-    git_wd = git.GitWorkdir(os.fspath(signed_commit_wd.cwd))
+    git_wd = git.GitWorkdir(signed_commit_wd.cwd)
     assert git_wd.get_head_date() == today
 
 
 @pytest.mark.parametrize(
-    "expected, from_data",
+    ("expected", "from_data"),
     [
         (
             "1.0",
@@ -528,15 +549,12 @@ def test_git_getdate_signed_commit(signed_commit_wd: WorkDir) -> None:
 )
 @pytest.mark.filterwarnings("ignore:git archive did not support describe output")
 def test_git_archival_to_version(expected: str, from_data: dict[str, str]) -> None:
-    config = Configuration()
+    config = Configuration(
+        version_scheme="guess-next-dev", local_scheme="node-and-date"
+    )
     version = archival_to_version(from_data, config=config)
     assert version is not None
-    assert (
-        format_version(
-            version, version_scheme="guess-next-dev", local_scheme="node-and-date"
-        )
-        == expected
-    )
+    assert format_version(version) == expected
 
 
 @pytest.mark.issue("https://github.com/pypa/setuptools_scm/issues/727")
@@ -549,6 +567,8 @@ def test_git_archival_node_missing_no_version() -> None:
 def test_git_archival_from_unfiltered() -> None:
     config = Configuration()
 
-    with pytest.warns(UserWarning, match="unexported git archival found"):
+    with pytest.warns(
+        UserWarning, match=r"unprocessed git archival found \(no export subst applied\)"
+    ):
         version = archival_to_version({"node": "$Format:%H$"}, config=config)
     assert version is None
